@@ -24,6 +24,7 @@
  * ```
  */
 #![allow(clippy::field_reassign_with_default)]
+use std::borrow::Cow;
 use std::collections::HashMap;
 use std::env;
 use std::error;
@@ -76,7 +77,7 @@ impl Shippo {
         Shippo::new(token)
     }
 
-    fn request<B>(&self, method: Method, path: &str, body: B, query: Option<Vec<(&str, String)>>) -> Request
+    fn request<B>(&self, method: Method, path: &str, body: B, query: Option<Vec<(String, String)>>) -> Request
     where
         B: Serialize,
     {
@@ -153,6 +154,67 @@ impl Shippo {
         Ok(resp.json().await.unwrap())
     }
 
+    /// List the carrier accounts.
+    /// FROM: https://goshippo.com/docs/reference#carrier-accounts
+    pub async fn list_carrier_accounts(&self) -> Result<Vec<CarrierAccount>, APIError> {
+        // Build the request.
+        let mut request = self.request(Method::GET, "carrier_accounts", (), None);
+
+        let mut resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                })
+            }
+        };
+
+        let mut r: CarrierAccountsAPIResponse = resp.json().await.unwrap();
+        let mut carrier_accounts = r.carrier_accounts;
+        let mut page = r.next;
+
+        // Paginate if we should.
+        // TODO: make this more DRY
+        while !page.is_empty() {
+            let url = Url::parse(&page).unwrap();
+            let pairs: Vec<(Cow<'_, str>, Cow<'_, str>)> = url.query_pairs().collect();
+            let mut new_pairs: Vec<(String, String)> = Vec::new();
+            for (a, b) in pairs {
+                let sa = a.into_owned();
+                let sb = b.into_owned();
+                new_pairs.push((sa, sb));
+            }
+
+            request = self.request(Method::GET, "carrier_accounts", (), Some(new_pairs));
+
+            resp = self.client.execute(request).await.unwrap();
+            match resp.status() {
+                StatusCode::OK => (),
+                s => {
+                    return Err(APIError {
+                        status_code: s,
+                        body: resp.text().await.unwrap(),
+                    })
+                }
+            };
+
+            // Try to deserialize the response.
+            r = resp.json().await.unwrap();
+
+            carrier_accounts.append(&mut r.carrier_accounts);
+
+            if !r.next.is_empty() && r.next != page {
+                page = r.next;
+            } else {
+                page = "".to_string();
+            }
+        }
+
+        Ok(carrier_accounts)
+    }
+
     /// Get a shipment.
     /// FROM: https://goshippo.com/docs/reference#shipments-retrieve
     pub async fn get_shipment(&self, id: &str) -> Result<Shipment, APIError> {
@@ -175,9 +237,9 @@ impl Shippo {
 
     /// Create a pickup.
     /// FROM: https://goshippo.com/docs/reference#pickups-create
-    pub async fn create_pickup(&self, np: NewPickup) -> Result<Pickup, APIError> {
+    pub async fn create_pickup(&self, np: &NewPickup) -> Result<Pickup, APIError> {
         // Build the request.
-        let request = self.request(Method::POST, "pickups", np, None);
+        let request = self.request(Method::POST, "pickups/", np, None);
 
         let resp = self.client.execute(request).await.unwrap();
         match resp.status() {
@@ -253,7 +315,7 @@ impl Shippo {
         let status = &resp.status();
 
         match &resp.json::<Transaction>().await {
-            Ok(v) => return Ok(v.clone()),
+            Ok(v) => Ok(v.clone()),
             Err(e) => {
                 return Err(APIError {
                     status_code: *status,
@@ -374,6 +436,18 @@ pub struct APIResponse {
     pub shipments: Vec<Shipment>,
 }
 
+/// The data type for an API response for carrier accounts.
+#[derive(Clone, Debug, Default, Serialize, Deserialize)]
+pub struct CarrierAccountsAPIResponse {
+    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    pub next: String,
+    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    pub previous: String,
+
+    #[serde(default, skip_serializing_if = "Vec::is_empty", alias = "results")]
+    pub carrier_accounts: Vec<CarrierAccount>,
+}
+
 /// The data type for a transactions API response.
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
 pub struct TransactionsAPIResponse {
@@ -444,6 +518,35 @@ pub struct Shipment {
     pub test: bool,
 }
 
+/// The data type for a carrier account.
+/// FROM: https://goshippo.com/docs/reference#carrier-accounts
+#[derive(Clone, Debug, Default, JsonSchema, Serialize, Deserialize)]
+pub struct CarrierAccount {
+    /// Unique identifier of the given CarrierAccount object.
+    pub object_id: String,
+    /// Username of the user who created the CarrierAccount object.
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub object_owner: String,
+    /// Name of the carrier. Please check the carrier accounts tutorial page
+    /// for all supported carrier names.
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub carrier: String,
+    /// Unique identifier of the account.
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub account_id: String,
+    /// An array of additional parameters for the account, such as e.g. password or token.
+    #[serde(default, skip_serializing_if = "HashMap::is_empty")]
+    pub parameters: HashMap<String, String>,
+    /// Determines whether the account is active. When creating a shipment, if
+    /// no carrier_accounts are explicitly passed Shippo will query all carrier
+    /// accounts that have this field set. By default, this is set to True.
+    #[serde(default)]
+    pub active: bool,
+    /// Indicates whether the object has been created in test mode.
+    #[serde(default)]
+    pub test: bool,
+}
+
 /// The data type for an address.
 /// FROM: https://goshippo.com/docs/reference#addresses
 #[derive(Clone, Debug, Default, JsonSchema, Serialize, Deserialize)]
@@ -456,7 +559,7 @@ pub struct Address {
     /// Incomplete addresses have failed one or multiple validations.
     /// Incomplete Addresses are eligible for requesting rates but lack at
     /// least one required value for purchasing labels.
-    #[serde(default)]
+    #[serde(default, skip_serializing_if = "is_false")]
     pub is_complete: bool,
     /// First and Last Name of the addressee
     #[serde(default, skip_serializing_if = "String::is_empty")]
@@ -501,13 +604,17 @@ pub struct Address {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub email: String,
     /// Indicates whether the object has been created in test mode.
-    #[serde(default)]
+    #[serde(default, skip_serializing_if = "is_false")]
     pub test: bool,
     /// object that contains information regarding if an address had been validated or not. Also
     /// contains any messages generated during validation. Children keys are is_valid(boolean) and
     /// messages(array).
-    #[serde(default)]
-    pub validation_results: ValidationResults,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub validation_results: Option<ValidationResults>,
+}
+
+fn is_false(t: &bool) -> bool {
+    !t
 }
 
 impl Address {
